@@ -86,6 +86,8 @@
 #ifndef _PBS_IFL_DEF
 #define _PBS_IFL_DEF
 
+#include <sys/socket.h>
+
 /* Attribute Names used by user commands */
 
 #define ATTR_a "Execution_Time"
@@ -298,6 +300,7 @@
   #define ATTR_authusers       "authorized_users"
 #endif
 #define ATTR_moabarraycompatible "moab_array_compatible"
+#define ATTR_nomailforce       "no_mail_force"
 /* additional node "attributes" names */
 
 #define ATTR_NODE_state          "state"
@@ -631,6 +634,9 @@ extern int
 
 extern int pbs_gpumode(int connect, char *node, char *gpuid, int gpumode);
 
+extern int trq_set_preferred_network_interface(char *if_name, struct sockaddr *preferred_addr);
+extern char *trq_get_if_name(void);
+
 #endif /* _PBS_IFL_DEF */
 
 /*  end of pbs_ifl.h  */
