@@ -86,6 +86,11 @@
 
 #include <pbs_config.h>   /* the master config generated by configure */
 
+#include <sys/ioctl.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <net/if.h>
+#include <arpa/inet.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <stdio.h>
@@ -96,9 +101,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
 #include <sys/un.h>
-#include <netinet/in.h>
 #include <sys/param.h>
 #if HAVE_SYS_UCRED_H
 #include <sys/ucred.h>
@@ -424,7 +427,15 @@ int PBSD_munge_authenticate(
         {
         /* read failed */
         fprintf(stderr, "error reading pipe in PBSD_munge_authenticate: errno = %d\n", errno);
-        return(-1);
+        close(fd_pipe[0]);
+        return(PBSE_SYSTEM);
+        }
+
+      /* if we got no bytes back then Munge may not be installed etc. */
+      if(total_bytes_read == 0)
+        {
+        close(fd_pipe[0]);
+        return(PBSE_MUNGE_NOT_FOUND);
         }
 
       /* We got the certificate. Now make the PBS_BATCH_AltAuthenUser request */
@@ -439,6 +450,7 @@ int PBSD_munge_authenticate(
       if(rc == -1)
         {
         fprintf(stderr, "getsockname failed: %d\n", errno);
+        close(fd_pipe[0]);
         return(-1);
         }
 
@@ -474,6 +486,7 @@ int PBSD_munge_authenticate(
     exit(0);
     }
 
+  close(fd_pipe[0]);
   return(0);  
   }
 #endif /* ifdef MUNGE_AUTH */
@@ -495,6 +508,8 @@ int PBSD_authenticate(
   int    j;
   FILE *piff;
   char  *ptr;
+  char  *path_copy;
+  char  *token;
 
   struct stat buf;
 
@@ -526,18 +541,26 @@ int PBSD_authenticate(
         {
         ptr = strtok(ptr, ":");
 
-        while (ptr != NULL)
+        if ((path_copy = strdup(ptr)) == NULL)
           {
-          snprintf(iffpath, sizeof(iffpath), "%s/pbs_iff",
-                   ptr);
+          fprintf(stderr,"Cannot allocate memory, FAILURE");
+
+          return(-1);
+          }
+
+        token = strtok(path_copy, ":");
+
+        while (token != NULL)
+          {
+          snprintf(iffpath, sizeof(iffpath), "%s/pbs_iff", token);
 
           rc = stat(iffpath, &buf);
 
           if (rc != -1)
             break;
 
-          ptr = strtok(NULL, ":");
-          }  /* END while (ptr != NULL) */
+          token = strtok(NULL, ":");
+          }  /* END while (token != NULL) */
         }    /* END if ((ptr = getenv("PATH")) != NULL) */
 
       if (rc == -1)
@@ -675,6 +698,73 @@ ssize_t    send_unix_creds(int sd)
 
 #endif /* END ENABLE_UNIX_SOCKETS */
 
+/*
+ * Set a preferred outbound interface for communication
+ * 
+ * The current solution uses a Linux ioctl call to
+ * find the named interfaces address. We will need
+ * to conditionally compile this for other platforms
+ *
+ * Return 0 on success. Non-zero on failure
+ */
+#define MAX_IFS 64
+        
+int trq_set_preferred_network_interface(char *if_name, struct sockaddr *preferred_addr)
+  {
+	struct ifreq *ifr, *ifend;
+	struct ifreq ifreqx;
+	struct ifconf ifc;
+	struct ifreq ifs[MAX_IFS];
+	int sockfd;
+
+	/* make sure we have a valid name for the interface */
+  if(if_name == NULL)
+		return(PBSE_IVALREQ);	
+
+  if(preferred_addr == NULL)
+    {
+    return(PBSE_IVALREQ);
+    }
+    
+  memset(preferred_addr, 0, sizeof(struct sockaddr));
+
+	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
+	if(sockfd < 0)
+    {
+    /* We can't do this without a socket */
+    return(PBSE_SYSTEM);
+    }
+
+	ifc.ifc_len = sizeof(ifs);
+	ifc.ifc_req = ifs;
+	if (ioctl(sockfd, SIOCGIFCONF, &ifc) < 0)
+    {
+    return(PBSE_SYSTEM);
+    }
+
+	/* We will walk through all the interfaces until we find the one we are looking for */
+  ifend = ifs + (ifc.ifc_len / sizeof(struct ifreq));
+  for (ifr = ifc.ifc_req; ifr < ifend; ifr++)
+    {
+    if (ifr->ifr_addr.sa_family == AF_INET)
+      {
+      if(!strncmp(if_name, ifr->ifr_name, IF_NAMESIZE))
+        {
+        strncpy(ifreqx.ifr_name, ifr->ifr_name,sizeof(ifreqx.ifr_name));
+
+        if(ioctl(sockfd, SIOCGIFADDR, &ifreqx) < 0)
+        	{
+        	close(sockfd);
+        	return(PBSE_SYSTEM);
+          }
+        /* get the address */
+        memcpy(preferred_addr, &ifreqx.ifr_ifru.ifru_addr, sizeof(struct sockaddr));
+        }
+      }
+    }
+    return(PBSE_NONE);
+  }
+
 
 /* returns socket descriptor or negative value (-1) on failure */
 
@@ -690,12 +780,14 @@ int pbs_original_connect(
 
   {
   struct sockaddr_in server_addr;
-
+  char *if_name;
   struct hostent *hp;
   int out;
   int i;
   int auth;
+  int rc;
 
+  struct sockaddr preferred_addr; /* set if TRQ_IFNAME set in torque.cfg */
   struct passwd *pw;
   int use_unixsock = 0;
 #ifdef ENABLE_UNIX_SOCKETS
@@ -905,9 +997,38 @@ int pbs_original_connect(
                 strerror(errno));
         }
 
-      return(-1);
+      return(PBSE_SYSTEM);
       }
 
+    if_name = trq_get_if_name();
+    if(if_name)
+      {
+      rc = trq_set_preferred_network_interface(if_name, &preferred_addr);
+      if(rc != PBSE_NONE)
+        {
+        fprintf(stderr, "could not set preferred network interface (%s): %d\n",
+                if_name, rc);
+        if(if_name)
+          free(if_name);
+        return(rc);
+        }
+
+      rc = bind(connection[out].ch_socket, &preferred_addr, sizeof(struct sockaddr));
+      if(rc < 0)
+        {
+        fprintf(stderr, "ERROR: could not bind preferred network interface (%s): errno: %d",
+                if_name, errno);
+        if(if_name)
+          free(if_name);
+        return(PBSE_SYSTEM);
+        }
+      }
+
+    /* we are done with if_name at this point. trq_get_if_name allocated
+       space for it. We need to free it */
+    if(if_name)
+      free(if_name);
+
     memcpy((char *)&server_addr.sin_addr, hp->h_addr_list[0], hp->h_length);
 
     server_addr.sin_port = htons(server_port);
@@ -1043,7 +1164,7 @@ int pbs_disconnect(
 
   {
   int  sock;
-  static char x[THE_BUF_SIZE / 4];
+  int  rc;
 
   /* send close-connection message */
 
@@ -1051,42 +1172,18 @@ int pbs_disconnect(
 
   DIS_tcp_setup(sock);
 
-  if ((encode_DIS_ReqHdr(sock, PBS_BATCH_Disconnect, pbs_current_user) == 0) &&
-      (DIS_tcp_wflush(sock) == 0))
+  rc = encode_DIS_ReqHdr(sock, PBS_BATCH_Disconnect, pbs_current_user);
+  if(rc)
     {
-    int atime;
-
-    struct sigaction act;
-
-    struct sigaction oldact;
-
-    /* set alarm to break out of potentially infinite read */
-
-/*    act.sa_handler = SIG_IGN; */
-    act.sa_handler = empty_alarm_handler;  /* need SOME handler or blocking read never gets interrupted */
-
-    sigemptyset(&act.sa_mask);
-    act.sa_flags = 0;
-    sigaction(SIGALRM, &act, &oldact);
-
-    atime = alarm(pbs_tcp_timeout);
-
-    while (1)
-      {
-      /* wait for server to close connection */
-
-      /* NOTE:  if read of 'sock' is blocking, request below may hang forever
-         -- hence the signal handler above */
-
-      if (read(sock, &x, sizeof(x)) < 1)
-        break;
-      }
-
-    alarm(atime);
-
-    sigaction(SIGALRM, &oldact, NULL);
+    return(rc);
     }
 
+   rc = DIS_tcp_wflush(sock);
+   if(rc)
+     {
+     return(rc);
+     }
+    
   close(sock);
 
   if (connection[connect].ch_errtxt != (char *)NULL)
