@@ -583,6 +583,15 @@ void scan_for_exiting(void)
           log_buffer);
         }
 
+      /* This is quasi odd. If we are in an EXITED substate then we
+         already sent the obit to the server and it replied. But
+         we have not received a PBS_BATCH_DeleteJob request from the 
+         server. So we will delete the job ourself. The server
+         does not look for a return code from PBS_BATCH_DeleteJob
+         anyway */
+      if(pjob->ji_qs.ji_substate == JOB_SUBSTATE_EXITED)
+        mom_deljob(pjob);
+
       continue;
       }
 
@@ -987,8 +996,12 @@ int post_epilogue(
 
   strcpy(preq->rq_ind.rq_jobobit.rq_jid, pjob->ji_qs.ji_jobid);
 
-  preq->rq_ind.rq_jobobit.rq_status =
-    pjob->ji_qs.ji_un.ji_momt.ji_exitstat;
+  if (pjob->ji_job_is_being_rerun == TRUE)
+    {
+    pjob->ji_qs.ji_un.ji_momt.ji_exitstat = 0;
+    }
+ 
+  preq->rq_ind.rq_jobobit.rq_status = pjob->ji_qs.ji_un.ji_momt.ji_exitstat;
 
   CLEAR_HEAD(preq->rq_ind.rq_jobobit.rq_attr);
 
@@ -1758,6 +1771,11 @@ void init_abort_jobs(
     exit(1);
     }
 
+  if (multi_mom)
+    {
+    momport = pbs_rm_port;
+    }
+
   while ((pdirent = readdir(dir)) != NULL)
     {
     if ((i = strlen(pdirent->d_name)) <= job_suf_len)
@@ -1768,7 +1786,7 @@ void init_abort_jobs(
     if (strcmp(psuffix, job_suffix))
       continue;
 
-    pj = job_recov(pdirent->d_name);
+    pj = job_recov(pdirent->d_name, momport);
 
     if (pj == NULL)
       {
@@ -1783,6 +1801,30 @@ void init_abort_jobs(
 
       continue;
       }
+     
+    /* code moved to here because even when we're canceling jobs, if there is a 
+     * user epilogue we'll attempt to become the user, so if ji_grpcache is 
+     * NULL then we'll get a crash */
+    if (pj->ji_grpcache == NULL)
+      {
+      DBPRT(("init_abort_jobs: setting grpcache for job %s\n",
+        pj->ji_qs.ji_jobid));
+      
+      if (check_pwd(pj) == NULL)
+        {
+        /* somehow a job that was legally executing (had a password entry)
+         * no longer has a password entry?? */
+        snprintf(log_buffer, sizeof(log_buffer),
+          "job %s no longer has valid password entry - deleting",
+          pj->ji_qs.ji_jobid);
+        
+        log_err(-1, id, log_buffer);
+        
+        mom_deljob(pj);
+
+        continue;
+        }
+      }
 
     /* PW:  mpiexec patch - set the globid so mom does not coredump in response to tm_spawn */
 
@@ -1961,10 +2003,6 @@ void init_abort_jobs(
          will not try to kill the running processes for this job */
       pj->ji_qs.ji_substate = JOB_SUBSTATE_NOTERM_REQUE;
 
-      if (multi_mom)
-        {
-        momport = pbs_rm_port;
-        }
 
       job_save(pj, SAVEJOB_QUICK, momport);
 
@@ -2004,24 +2042,11 @@ void init_abort_jobs(
 
       if (mom_do_poll(pj) && (recover == JOB_RECOV_RUNNING))
         append_link(&mom_polljobs, &pj->ji_jobque, pj);
-
-      if (pj->ji_grpcache == NULL)
-        {
-        DBPRT(("init_abort_jobs: setting grpcache for job %s\n",
-               pj->ji_qs.ji_jobid));
-        check_pwd(pj);
-        }
       }
     }    /* while ((pdirent = readdir(dir)) != NULL) */
 
   closedir(dir);
 
-#if defined(PENABLE_LINUX26_CPUSETS)
-  /* Create the top level torque cpuset if it doesn't already exist. */
-  initialize_root_cpuset();
-
-#endif
-
   return;
   }  /* END init_abort_jobs() */
 
