@@ -111,6 +111,7 @@
 #include "log.h"
 #include "mom_mach.h"
 #include "resmon.h"
+#include "utils.h"
 #include "../rm_dep.h"
 
 /*
@@ -137,7 +138,7 @@
 **  valid_user is user name valid..
 */
 
-static char ident[] = "@(#) hpux10/$RCSfile$ $Revision: 3230 $";
+static char ident[] = "@(#) hpux10/$RCSfile$ $Revision: 4932 $";
 
 #ifndef TRUE
 #define FALSE 0
@@ -2140,7 +2141,7 @@ valid_user(struct rm_attribute *attrib)
     return NULL;
     }
 
-  p = getpwnam(attrib -> a_value);
+  p = getpwnam_ext(attrib -> a_value);
 
   if (p)
     {
