@@ -118,6 +118,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <netdb.h>
+#include <pthread.h>
 
 #include "pbs_ifl.h"
 #include "list_link.h"
@@ -134,6 +135,7 @@
 #include "acct.h"
 #include "net_connect.h"
 #include "portability.h"
+#include "threadpool.h"
 
 
 #ifndef TRUE
@@ -145,7 +147,8 @@ int conn_qsub(char *, long, char *);
 void job_purge(job *);
 
 /* External functions */
-extern void mom_checkpoint_delete_files(job *pjob);
+extern void mom_checkpoint_delete_files(job_file_delete_info *);
+extern void mom_server_all_update_gpustat(void);
 
 #if IBM_SP2==2  /* IBM SP PSSP 3.1 */
 void unload_sp_switch(job *pjob);
@@ -160,11 +163,14 @@ static void job_init_wattr(job *);
 
 /* Global Data items */
 
+extern char tmpdir_basename[];	/* for TMPDIR */
 extern gid_t pbsgroup;
+extern uid_t pbsuser;
 extern char *msg_abt_err;
 extern char *path_jobs;
 extern char *path_spool;
 extern char *path_aux;
+extern char *msg_err_purgejob;
 extern char  server_name[];
 extern time_t time_now;
 extern int   LOGLEVEL;
@@ -174,6 +180,11 @@ extern tlist_head svr_alljobs;
 
 void nodes_free(job *);
 int TTmpDirName(job *, char *);
+extern int thread_unlink_calls;
+
+extern void MOMCheckRestart(void);
+extern int delete_cpuset(char *);
+
 
 extern int multi_mom;
 extern int pbs_rm_port;
@@ -245,9 +256,10 @@ int remtree(
   DIR  *dir;
 
   struct dirent *pdir;
-  char  namebuf[MAXPATHLEN], *filnam;
-  int  i;
-  int  rtnv = 0;
+  char           namebuf[MAXPATHLEN];
+  char          *filnam;
+  int            i;
+  int            rtnv = 0;
 #if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64) && defined(LARGEFILE_WORKS)
 
   struct stat64 sb;
@@ -317,11 +329,9 @@ int remtree(
         {
         if (errno != ENOENT)
           {
-          sprintf(log_buffer, "unlink failed on %s",
-                  namebuf);
-
+          sprintf(log_buffer, "unlink failed on %s", namebuf);
           log_err(errno, id, log_buffer);
-
+          
           rtnv = -1;
           }
         }
@@ -356,10 +366,8 @@ int remtree(
     }
   else if (unlink(dirname) < 0)
     {
-    sprintf(log_buffer, "unlink failed on %s",
-            dirname);
-
-    log_err(errno, id, log_buffer);
+    snprintf(log_buffer,sizeof(log_buffer),"unlink failed on %s",dirname);
+    log_err(errno,id,log_buffer);
 
     rtnv = -1;
     }
@@ -453,8 +461,7 @@ int conn_qsub(
  * Returns: pointer to structure or null is space not available.
  */
 
-job *
-job_alloc(void)
+job *job_alloc(void)
 
   {
   job *pj;
@@ -487,11 +494,12 @@ job_alloc(void)
   pj->ji_globid = NULL;
   pj->ji_stdout = 0;
   pj->ji_stderr = 0;
+  pj->ji_qs.ji_un.ji_momt.ji_exitstat = 0;
+  pj->ji_job_is_being_rerun = 0;
 
   pj->ji_momhandle = -1;  /* mark mom connection invalid */
 
   /* set the working attributes to "unspecified" */
-
   job_init_wattr(pj);
 
   return(pj);
@@ -562,11 +570,15 @@ void job_free(
  * job_unlink_file - unlink file, but drop root credentials before
  * doing this to avoid removing objects that aren't belong to the user.
  */
+
 int job_unlink_file(
+
   job *pjob,		/* I */
   const char *name)	/* I */
+
   {
-  int saved_errno = 0, result = 0;
+  int saved_errno = 0;
+  int result = 0;
   uid_t uid = geteuid();
   gid_t gid = getegid();
 
@@ -593,10 +605,13 @@ int job_unlink_file(
   }  /* END job_unlink_file() */
 
 
+
+
 /*
  * job_init_wattr - initialize job working attribute array
  * set the types and the "unspecified value" flag
  */
+
 static void job_init_wattr(
 
   job *pj)
@@ -604,7 +619,7 @@ static void job_init_wattr(
   {
   int i;
 
-  for (i = 0;i < JOB_ATR_LAST;i++)
+  for (i = 0; i < JOB_ATR_LAST; i++)
     {
     clear_attr(&pj->ji_wattr[i], &job_attr_def[i]);
     }
@@ -615,6 +630,151 @@ static void job_init_wattr(
 
 
 
+void *delete_job_files(
+
+  void *vp)
+
+  {
+  static char          *id = "remove_dir";
+  job_file_delete_info *jfdi = (job_file_delete_info *)vp;
+  char                  namebuf[MAXPATHLEN+1];
+  int                   rc = 0;
+
+  if (jfdi->has_temp_dir == TRUE)
+    {
+    if (tmpdir_basename[0] == '/')
+      {
+      snprintf(namebuf, sizeof(namebuf), "%s/%s", tmpdir_basename, jfdi->jobid);
+      sprintf(log_buffer, "removing transient job directory %s", namebuf);
+
+      log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,jfdi->jobid,log_buffer);
+
+      if ((setegid(jfdi->gid) == -1) ||
+          (seteuid(jfdi->uid) == -1))
+        {
+        /* FAILURE */
+        rc = -1;
+        }
+      else
+        {
+        rc = remtree(namebuf);
+        
+        seteuid(pbsuser);
+        setegid(pbsgroup);
+        }
+      }
+
+    if ((rc != 0) &&
+        (LOGLEVEL >= 5))
+      {
+      snprintf(log_buffer,sizeof(log_buffer),
+        "recursive remove of job transient tmpdir %s failed",
+        namebuf);
+
+      log_err(errno,id,log_buffer);
+      }
+    } /* END code to remove temp dir */
+
+#ifdef PENABLE_LINUX26_CPUSETS
+  /* Delete the cpuset for the job. */
+  delete_cpuset(jfdi->jobid);
+#endif /* PENABLE_LINUX26_CPUSETS */
+
+  /* delete the node file and gpu file */
+  if (jfdi->has_node_file == TRUE)
+    {
+    sprintf(namebuf,"%s/%s", path_aux, jfdi->jobid);
+    unlink(namebuf);
+    
+    sprintf(namebuf, "%s/%sgpu", path_aux, jfdi->jobid);
+    unlink(namebuf);
+    } /* END code to delete node and gpu files */
+
+  /* delete script file */
+  if (multi_mom)
+    {
+    snprintf(namebuf, sizeof(namebuf), "%s%s%d%s",
+      path_jobs,
+      jfdi->prefix,
+      pbs_rm_port,
+      JOB_SCRIPT_SUFFIX);
+    }
+  else
+    {
+    snprintf(namebuf, sizeof(namebuf), "%s%s%s",
+      path_jobs,
+      jfdi->prefix,
+      JOB_SCRIPT_SUFFIX);
+    }
+  
+  if (unlink(namebuf) < 0)
+    {
+    if (errno != ENOENT)
+      log_err(errno,id,msg_err_purgejob);
+    }
+  else
+    {
+    snprintf(log_buffer,sizeof(log_buffer),"removed job script");
+    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,jfdi->jobid,log_buffer);
+    }
+  
+  /* delete job task directory */
+  if (multi_mom)
+    {
+    snprintf(namebuf,sizeof(namebuf),"%s%s%d%s",
+      path_jobs,
+      jfdi->prefix,
+      pbs_rm_port,
+      JOB_TASKDIR_SUFFIX);
+    }
+  else
+    {
+    snprintf(namebuf,sizeof(namebuf),"%s%s%s",
+      path_jobs,
+      jfdi->prefix,
+      JOB_TASKDIR_SUFFIX);
+    }
+  
+  remtree(namebuf);
+  
+  mom_checkpoint_delete_files(jfdi);
+  
+  /* delete job file */
+  if (multi_mom)
+    {
+    snprintf(namebuf,sizeof(namebuf),"%s%s%d%s",
+      path_jobs,
+      jfdi->prefix,
+      pbs_rm_port,
+      JOB_FILE_SUFFIX);
+    }
+  else
+    {
+    snprintf(namebuf,sizeof(namebuf),"%s%s%s",
+      path_jobs,
+      jfdi->prefix,
+      JOB_FILE_SUFFIX);
+    }
+  
+  if (unlink(namebuf) < 0)
+    {
+    if (errno != ENOENT)
+      log_err(errno,id,msg_err_purgejob);
+    }
+  else if (LOGLEVEL >= 6)
+    {
+    snprintf(log_buffer,sizeof(log_buffer),"remove job file");
+    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,jfdi->jobid,log_buffer);
+    }
+  
+  free(jfdi);
+
+  return(NULL);
+  } /* END delete_files() */
+
+
+
+
 
 /*
  * job_purge - purge job from system
@@ -627,188 +787,76 @@ static void job_init_wattr(
 
 void job_purge(
 
-  job *pjob)  /* I (modified) */
+  job *pjob) /* M */
 
   {
-  static char   id[] = "job_purge";
+  static char           id[] = "job_purge";
+  job_file_delete_info *jfdi;
 
-  char          namebuf[MAXPATHLEN + 1];
-  char          portname[MAXPATHLEN + 1];
-  extern char  *msg_err_purgejob;
-  int           rc;
+  jfdi = malloc(sizeof(job_file_delete_info));
 
-  extern void MOMCheckRestart(void);
+  if (jfdi == NULL)
+    {
+    log_err(ENOMEM,id,"No space to allocate info for job file deletion");
+    return;
+    }
 
+  /* initialize struct information */
   if (pjob->ji_flags & MOM_HAS_TMPDIR)
     {
-    if (TTmpDirName(pjob, namebuf))
-      {
-      sprintf(log_buffer, "removing transient job directory %s",
-        namebuf);
-
-      log_record(
-        PBSEVENT_DEBUG,
-        PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid,
-        log_buffer);
-
-      if ((setegid(pjob->ji_qs.ji_un.ji_momt.ji_exgid) == -1) ||
-          (seteuid(pjob->ji_qs.ji_un.ji_momt.ji_exuid) == -1))
-        {
-        /* FAILURE */
-
-        return;
-        }
-
-      rc = remtree(namebuf);
-
-      seteuid(0);
-      setegid(pbsgroup);
-
-      if ((rc != 0) && (LOGLEVEL >= 5))
-        {
-        sprintf(log_buffer,
-          "recursive remove of job transient tmpdir %s failed",
-          namebuf);
-
-        log_err(errno, "recursive (r)rmdir",
-          log_buffer);
-        }
-
-      pjob->ji_flags &= ~MOM_HAS_TMPDIR;
-      }
+    jfdi->has_temp_dir = TRUE;
+    pjob->ji_flags &= ~MOM_HAS_TMPDIR;
     }
-
-#ifdef PENABLE_LINUX26_CPUSETS
-
-  if (use_cpusets(pjob) == TRUE)
-    {
-    extern void delete_cpuset(char *);
-
-    /* Delete the cpuset for the job. */
-
-    delete_cpuset(pjob->ji_qs.ji_jobid);
-    }
-
-#endif /* PENABLE_LINUX26_CPUSETS */
-
-  /* delete the nodefile if still hanging around */
-
+  else
+    jfdi->has_temp_dir = FALSE;
+  
   if (pjob->ji_flags & MOM_HAS_NODEFILE)
     {
-    char file[MAXPATHLEN + 1];
-
-    sprintf(file,"%s/%s",
-      path_aux,
-      pjob->ji_qs.ji_jobid);
-
-    unlink(file);
-
+    jfdi->has_node_file = TRUE;
     pjob->ji_flags &= ~MOM_HAS_NODEFILE;
     }
+  else
+    jfdi->has_node_file = FALSE;
 
+  strcpy(jfdi->jobid,pjob->ji_qs.ji_jobid);
+  strcpy(jfdi->prefix,pjob->ji_qs.ji_fileprefix);
+  
+  if ((pjob->ji_wattr[JOB_ATR_checkpoint_dir].at_flags & ATR_VFLAG_SET) &&
+      (pjob->ji_wattr[JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET))
+    jfdi->checkpoint_dir = strdup(pjob->ji_wattr[JOB_ATR_checkpoint_dir].at_val.at_str);
+  else
+    jfdi->checkpoint_dir = NULL;
+  
+  jfdi->gid = pjob->ji_qs.ji_un.ji_momt.ji_exgid;
+  jfdi->uid = pjob->ji_qs.ji_un.ji_momt.ji_exuid;
+  
+  if (thread_unlink_calls == TRUE)
+    enqueue_threadpool_request(delete_job_files,jfdi);
+  else
+    delete_job_files(jfdi);
+
+  /* remove this job from the global queue */
   delete_link(&pjob->ji_jobque);
-
   delete_link(&pjob->ji_alljobs);
-
+  
   if (LOGLEVEL >= 6)
     {
     sprintf(log_buffer,"removing job");
-
-    log_record(
-      PBSEVENT_DEBUG,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      log_buffer);
+    
+    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buffer);
     }
 
-  strcpy(namebuf,path_jobs); /* delete script file */
-
-  strcat(namebuf,pjob->ji_qs.ji_fileprefix);
-
-  if(multi_mom)
-    {
-    sprintf(portname, "%d", pbs_rm_port);
-    strcat(namebuf, portname);
-    }
-
-  strcat(namebuf,JOB_SCRIPT_SUFFIX);
-
-  if (unlink(namebuf) < 0)
-    {
-    if (errno != ENOENT)
-      log_err(errno, id, msg_err_purgejob);
-    }
-  else if (LOGLEVEL >= 6)
-    {
-    sprintf(log_buffer, "removed job script");
-
-    log_record(
-      PBSEVENT_DEBUG,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      log_buffer);
-    }
-
-
 #if IBM_SP2==2        /* IBM SP PSSP 3.1 */
   unload_sp_switch(pjob);
 
 #endif   /* IBM SP */
-  strcpy(namebuf, path_jobs);     /* job directory path */
-
-  strcat(namebuf, pjob->ji_qs.ji_fileprefix);
-
-  if(multi_mom)
-    {
-    sprintf(portname, "%d", pbs_rm_port);
-    strcat(namebuf, portname);
-    }
-
-  strcat(namebuf, JOB_TASKDIR_SUFFIX);
-
-  remtree(namebuf);
-
-  mom_checkpoint_delete_files(pjob);
-
-  strcpy(namebuf, path_jobs); /* delete job file */
-
-  strcat(namebuf, pjob->ji_qs.ji_fileprefix);
-
-  if(multi_mom)
-    {
-    sprintf(portname, "%d", pbs_rm_port);
-    strcat(namebuf, portname);
-    }
-
-  strcat(namebuf, JOB_FILE_SUFFIX);
-
-  if (unlink(namebuf) < 0)
-    {
-    if (errno != ENOENT)
-      log_err(errno, id, msg_err_purgejob);
-    }
-  else if (LOGLEVEL >= 6)
-    {
-    sprintf(log_buffer, "removed job file");
-
-    log_record(
-      PBSEVENT_DEBUG,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      log_buffer);
-    }
 
   job_free(pjob);
 
   /* if no jobs are left, check if MOM should be restarted */
-
   if (((job *)GET_NEXT(svr_alljobs)) == NULL)
     MOMCheckRestart();
-
-  return;
-  }  /* END job_purge() */
-
+  } /* END job_purge() */
 
 
 
