@@ -287,7 +287,7 @@ static pid_t fork_to_user(
     }
   else
     {
-    if ((pwdp = getpwnam(preq->rq_ind.rq_cpyfile.rq_user)) == NULL)
+    if ((pwdp = getpwnam_ext(preq->rq_ind.rq_cpyfile.rq_user)) == NULL)
       {
       if (MOMUNameMissing[0] == '\0')
         strncpy(MOMUNameMissing, preq->rq_ind.rq_cpyfile.rq_user, 64);
@@ -1276,8 +1276,8 @@ void req_gpuctrl(
   if (rc)
     {
     reply_ack(preq);
-    
-    /* 
+
+    /*
      * if we were successful changing the mode then we need to update the gpu
      * statuses
      */
@@ -2213,6 +2213,14 @@ void req_signaljob(
       log_buffer);
     }
 
+  if (preq->rq_extend != NULL)
+    {
+    if (!strcasecmp("rerun", preq->rq_extend))
+      {
+      pjob->ji_job_is_being_rerun = TRUE;
+      }
+    }
+
   if (!strcasecmp(sname, SIG_SUSPEND))
     {
     if (pjob->ji_qs.ji_substate != JOB_SUBSTATE_RUNNING)
@@ -2854,6 +2862,7 @@ static int del_files(
      */
 
 /*    if (del_dir) */
+    if (pair->fp_flag == JOBCKPFILE)
       {
       char *ptr;
       ptr = strrchr(path,'/');
