@@ -569,6 +569,9 @@ void account_jobstr(
   buf = (char *)malloc(bufSize+1);
   if (!buf)
     return;
+
+	memset(buf, 0, bufSize+1);
+
   acct_job(pjob, &buf, &bufSize);
 
   buf[bufSize] = '\0';
@@ -610,6 +613,8 @@ void account_jobend(
   if (!buf)
     return;
 
+	memset(buf, 0, bufSize);
+
   pb = acct_job(pjob, &buf, &bufSize);
 
   /* session */
