@@ -193,6 +193,7 @@ badplace *is_bad_dest(
   return(bp);
   }  /* END is_bad_dest() */
 
+
 /* int initialize_procct - set pjob->procct plus the resource
  * procct in the Resource_List
  *  
@@ -203,7 +204,7 @@ badplace *is_bad_dest(
  * Returns 0 on success. Non-zero on failure
  */ 
 int initialize_procct(job *pjob)
-  { 
+  {                \
   char id[] = "initialize_procct";
   resource     *pnodesp = NULL;
   resource_def *pnodes_def = NULL;
@@ -368,8 +369,7 @@ int remove_procct(job *pjob)
       pctdef->rs_free(&pctresc->rs_value);
 
   return(PBSE_NONE);
-  } /* END remove_procct */
-
+  }
 
 /*
  * default_router - basic function for "routing" jobs.
@@ -456,11 +456,6 @@ int default_router(
 
       case ROUTE_PERM_FAILURE: /* permanent failure */
 
-        rc = remove_procct(jobp);
-        if(rc != PBSE_NONE)
-          {
-          return(rc);
-          }
         add_dest(jobp);
 
         break;
@@ -497,9 +492,6 @@ int default_router(
   }  /* END default_router() */
 
 
-
-
-
 /*
  * job_route - route a job to another queue
  *
@@ -587,15 +579,15 @@ int job_route(
 
   qp = jobp->ji_qhdr;
 
-  if (qp->qu_attr[(int)QA_ATR_Started].at_val.at_long == 0)
+  if (qp->qu_attr[QA_ATR_Started].at_val.at_long == 0)
     {
     /* queue not started - no routing */
 
     return(0);
     }
 
-  if ((qp->qu_attr[(int)QA_ATR_MaxRun].at_flags & ATR_VFLAG_SET) &&
-      (qp->qu_attr[(int)QA_ATR_MaxRun].at_val.at_long <= qp->qu_njstate[JOB_STATE_TRANSIT]))
+  if ((qp->qu_attr[QA_ATR_MaxRun].at_flags & ATR_VFLAG_SET) &&
+      (qp->qu_attr[QA_ATR_MaxRun].at_val.at_long <= qp->qu_njstate[JOB_STATE_TRANSIT]))
     {
     /* max number of jobs being routed */
 
@@ -604,22 +596,22 @@ int job_route(
 
   /* what is the retry time and life time of a job in this queue */
 
-  if (qp->qu_attr[(int)QR_ATR_RouteRetryTime].at_flags & ATR_VFLAG_SET)
+  if (qp->qu_attr[QR_ATR_RouteRetryTime].at_flags & ATR_VFLAG_SET)
     {
     retry_time =
       (long)time_now +
-      qp->qu_attr[(int)QR_ATR_RouteRetryTime].at_val.at_long;
+      qp->qu_attr[QR_ATR_RouteRetryTime].at_val.at_long;
     }
   else
     {
     retry_time = (long)time_now + PBS_NET_RETRY_TIME;
     }
 
-  if (qp->qu_attr[(int)QR_ATR_RouteLifeTime].at_flags & ATR_VFLAG_SET)
+  if (qp->qu_attr[QR_ATR_RouteLifeTime].at_flags & ATR_VFLAG_SET)
     {
     life =
       jobp->ji_qs.ji_un.ji_routet.ji_quetime +
-      qp->qu_attr[(int)QR_ATR_RouteLifeTime].at_val.at_long;
+      qp->qu_attr[QR_ATR_RouteLifeTime].at_val.at_long;
     }
   else
     {
@@ -646,7 +638,7 @@ int job_route(
     return(0);
     }
 
-  if (qp->qu_attr[(int)QR_ATR_AltRouter].at_val.at_long == 0)
+  if (qp->qu_attr[QR_ATR_AltRouter].at_val.at_long == 0)
     {
     rc = default_router(jobp, qp, retry_time);
     if(rc == ROUTE_SUCCESS)
