@@ -1081,13 +1081,12 @@ static int process_host_name_part(
 
   memcpy((char *)&addr, hp->h_addr, hp->h_length);
 
-  if (hp->h_addr_list[1] == NULL)
+  if (hp->h_addr_list[0] != NULL)
     {
-    /* weren't given canonical name */
 
     char *hname;
 
-    if ((hp = gethostbyaddr(
+/*    if ((hp = gethostbyaddr(
                 (void *) & addr,
                 sizeof(struct in_addr),
                 hp->h_addrtype)) == NULL)
@@ -1104,7 +1103,7 @@ static int process_host_name_part(
       phostname = NULL;
 
       return(PBSE_UNKNODE);
-      }
+      }*/
 
     hname = (char *)strdup(hp->h_name); /* canonical name in theory */
 
@@ -2249,7 +2248,7 @@ int setup_nodes(void)
   static char id[] = "setup_nodes";
 
   FILE  *nin;
-  char   line[256];
+  char   line[MAXLINE << 4];
   char   note[MAX_NOTE+1];
   char  *nodename;
   char   propstr[256];
