@@ -390,6 +390,9 @@ void add_server_names_to_acl_hosts(void)
 
   struct attribute *patr = &server.sv_attr[SRV_ATR_acl_hosts];
 
+  memset(buffer, 0, PBS_MAXSERVERNAME+1);
+  memset(&temp, 0, sizeof(attribute));
+
   server_list_ptr = pbs_get_server_list();
   list_len = csv_length(server_list_ptr);
 
@@ -894,7 +897,12 @@ int pbsd_init(
 
   if (server.sv_attr[SRV_ATR_RecordJobInfo].at_val.at_long)
     {
-    job_log_open(job_log_file, path_jobinfo_log);
+    rc = job_log_open(job_log_file, path_jobinfo_log);
+    if(rc != 0)
+      {
+      fprintf(stderr, "Could not open job_logs \n");
+      return(-1);
+      }
     }
 
 
@@ -963,7 +971,7 @@ int pbsd_init(
       {
       /* recover queue */
 
-      if ((pque = que_recov(pdirent->d_name)) != NULL)
+      if ((pque = que_recov_xml(pdirent->d_name)) != NULL)
         {
         /* que_recov increments sv_numque */
 
@@ -1122,7 +1130,7 @@ int pbsd_init(
 
         if (!strcmp(psuffix, ".TA"))
           {
-          if ((pjob = job_recov(pdirent->d_name)) != NULL)
+          if ((pjob = job_recov(pdirent->d_name, 0)) != NULL)
             {
             pjob->ji_is_array_template = TRUE;
 
@@ -1140,7 +1148,7 @@ int pbsd_init(
         if (strcmp(psuffix, job_suffix))
           continue;
 
-        if ((pjob = job_recov(pdirent->d_name)) != NULL)
+        if ((pjob = job_recov(pdirent->d_name, 0)) != NULL)
           {
 
           if (DArrayAppend(&Array,pjob) == FAILURE)
