@@ -187,6 +187,7 @@ void req_holdjob(struct batch_request *preq);
 void req_checkpointjob(struct batch_request *preq);
 void req_messagejob(struct batch_request *preq);
 void req_modifyjob(struct batch_request *preq);
+void req_gpuctrl(struct batch_request *preq);
 #ifndef PBS_MOM
 void req_orderjob(struct batch_request *preq);
 void req_rescreserve(struct batch_request *preq);
@@ -325,7 +326,7 @@ void process_request(
 
   int                   rc;
 
-  struct batch_request *request;
+  struct batch_request *request = NULL;
 
   time_now = time(NULL);
 
@@ -543,10 +544,21 @@ void process_request(
       rc = 0;  /* bypass the authentication of the user--trust the client completely */
     else if (munge_on)
       {
-      /* If munge_on is true we will validate the connection later */
-      conn_credent[sfds].timestamp = time_now;
-      svr_conn[sfds].cn_authen = PBS_NET_CONN_AUTHENTICATED;
-      rc = 0;
+      /* If munge_on is true we will validate the connection now */
+      if ( request->rq_type == PBS_BATCH_AltAuthenUser)
+        {
+        rc = req_altauthenuser(request);
+        if (rc == PBSE_NONE)
+          {
+          conn_credent[sfds].timestamp = time_now;
+          svr_conn[sfds].cn_authen = PBS_NET_CONN_AUTHENTICATED;
+          }
+        return;
+        }
+      else
+        {
+        rc = authenticate_user(request, &conn_credent[sfds]);
+        }
       }
     else if (svr_conn[sfds].cn_authen != PBS_NET_CONN_AUTHENTICATED)
       rc = PBSE_BADCRED;
@@ -956,6 +968,12 @@ void dispatch_request(
 
       break;
 
+    case PBS_BATCH_GpuCtrl:
+
+      req_gpuctrl(request);
+
+      break;
+
     case PBS_BATCH_StatusJob:
 
       req_stat_job(request);
@@ -1018,9 +1036,6 @@ void dispatch_request(
       break;
 
     case PBS_BATCH_AltAuthenUser:
-      /* Use given authentication method to determine
-         if user is valid */
-       req_altauthenuser(request); 
 
       break;
 
@@ -1120,7 +1135,7 @@ struct batch_request *alloc_br(
 
   {
 
-  struct batch_request *req;
+  struct batch_request *req = NULL;
 
   req = (struct batch_request *)malloc(sizeof(struct batch_request));
 
