@@ -95,17 +95,13 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <ctype.h>
-#include <netdb.h>
-#include <signal.h>
 #include "libpbs.h"
-#include "log.h"
 #include "server_limits.h"
 #include "list_link.h"
 #include "attribute.h"
 #include "credential.h"
 #include "net_connect.h"
 #include "batch_request.h"
-#include "errno.h"
 
 #define SPACE 32 /* ASCII space character */
 
@@ -139,7 +135,7 @@ void req_connect(
     }
   else
     {
-    req_reject(PBSE_BADCRED, 0, preq, NULL, "Connection not authorized");
+    req_reject(PBSE_BADCRED, 0, preq, NULL, NULL);
     }
 
   return;
@@ -153,7 +149,7 @@ int get_encode_host(int s, char *munge_buf, struct batch_request *preq)
 
   /* ENCODE_HOST: is a keyword in the unmunge data that holds the host name */
   ptr = strstr(munge_buf, "ENCODE_HOST:");
-  if(!ptr)
+  if (!ptr)
     {
     req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not read unmunge data host");
     return(-1);
@@ -189,7 +185,7 @@ int get_UID(int s, char *munge_buf, struct batch_request *preq)
 
 
   ptr = strstr(munge_buf, "UID:");
-	if(!ptr)
+	if (!ptr)
 		{
 		req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not read unmunge data user");
 		return(-1);
@@ -215,44 +211,66 @@ int get_UID(int s, char *munge_buf, struct batch_request *preq)
 
   }
 
-
-
-
-int write_munge_temp_file(
-
-  struct batch_request *preq,          /* I */
-  char                 *mungeFileName) /* I */
-
+int unmunge_request(int s, struct batch_request *preq)
   {
-  int fd;
-  int cred_size;
+  time_t myTime;
+  struct timeval tv;
+  suseconds_t millisecs;
+  struct tm *timeinfo;
+  char mungeFileName[MAXPATHLEN + MAXNAMLEN+1];
+  int fd, newfd;
+  char buf[MUNGE_SIZE];
+  char munge_buf[MUNGE_SIZE];
   int bytes_written;
+  int cred_size;
+  pid_t pid;
+
+  int fd_pipe[2];
+  char execname[20]; /* for execvp. will be "munge" */
+  char *options[3];  /* argument list for execvp */
+  char com1[20];     /* first argument to execvp */
+  char com2[MAXPATHLEN + MAXNAMLEN+5];      /* second argument to execvp */
+  int bytes_read;
+  int total_bytes_read = 0;        
+  char *ptr; /* pointer to the current place to copy data into munge_buf */
   int rc;
 
-  if ((fd = open(mungeFileName, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0)
+  /* create a sudo randome file name */
+  gettimeofday(&tv, NULL);
+  myTime = tv.tv_sec;
+  timeinfo = localtime(&myTime);
+  millisecs = tv.tv_usec;
+  sprintf(mungeFileName, "%smunge-%d-%d-%d-%d", 
+  path_credentials, timeinfo->tm_hour, timeinfo->tm_min, 
+  timeinfo->tm_sec, (int)millisecs);
+
+  fd = open(mungeFileName, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
+  if(fd == -1)
     {
     req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not create temporary munge file");
-    return(-1);
+	  return(-1);
     }
 
-  if ((cred_size = strlen(preq->rq_ind.rq_authen.rq_cred)) == 0)
+  /* Write the munge credential to the newly created file */
+
+  cred_size = strlen(preq->rq_ind.rq_authen.rq_cred);
+  if(cred_size == 0)
     {
-    req_reject(PBSE_BADCRED, 0, preq, NULL, "munge credential invalid");
-    close(fd);
-    return(-1);
+    req_reject(PBSE_BADCRED, 0, preq, NULL, NULL);
+		close(fd);
+		return(-1);
     }
 
   bytes_written = write(fd, preq->rq_ind.rq_authen.rq_cred, cred_size);
-
-  if ((bytes_written == -1) || 
-      (bytes_written != cred_size))
-    {
-    req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not write credential to temporary munge file");
+  if(bytes_written == -1 || (bytes_written != cred_size))
+	  {
+	  req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not write credential to temporary munge file");
     close(fd);
     return(-1);
     }
 
-	if ((rc = fsync(fd)) < 0)
+	rc = fsync(fd);
+	if (rc < 0)
 		{
 		close(fd);
 		return(rc);
@@ -260,166 +278,104 @@ int write_munge_temp_file(
 
   close(fd);
 
-  return(PBSE_NONE);
-  } /* END write_munge_temp_file() */
-
-
-
-
-
-int pipe_and_read_unmunge(
-
-  char                 *mungeFileName, /* I */
-  struct batch_request *preq,          /* I */
-  int                   sock)          /* I */
-
-  {
-  static char *id = "pipe_and_read_unmunge";
-  char         munge_buf[MUNGE_SIZE << 4];
-  sigset_t     child_set;
-
-  FILE        *munge_pipe;
-  char        *ptr; /* pointer to the current place to copy data into munge_buf */
-  char         munge_command[MAXPATHLEN<<1];
-  int          bytes_read;
-  int          total_bytes_read = 0;
-  int          fd;
-  int          rc;
-  
-  snprintf(munge_command,sizeof(munge_command),
-    "unmunge --input=%s",
-    mungeFileName);
-  
-  if ((munge_pipe = popen(munge_command,"r")) == NULL)
+  /* For the child to run the unmunge utility on the file we just created.
+   	 The parent will read from the child and use the unmunged data to validate
+	 the user */
+  rc = pipe(fd_pipe);
+  if (rc == -1)
     {
-    /* FAILURE */
-    snprintf(log_buffer, sizeof(log_buffer),
-      "Unable to popen command '%s' for reading",
-      munge_command);
-    log_err(errno, id, log_buffer);
-    
     unlink(mungeFileName);
-    req_reject(PBSE_SYSTEM, 0, preq, NULL, "couldn't create pipe to unmunge");
+    req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not create pipe to unmunge");
     return(-1);
     }
-  
-  memset(munge_buf, 0, MUNGE_SIZE);
-  ptr = munge_buf;
-  
-  fd = fileno(munge_pipe);
-  
-  /* block SIGCHLD so we don't get messed up mid-loop */
-  sigemptyset(&child_set);
-  sigaddset(&child_set, SIGCHLD);
-  sigprocmask(SIG_BLOCK, &child_set, NULL);
 
-  while ((bytes_read = read(fd, ptr, MUNGE_SIZE)) > 0)
-    {
-    total_bytes_read += bytes_read;
-    ptr += bytes_read;
-    }
-
-  /* out of loop, we're good with SIGCHLD */
-  sigprocmask(SIG_UNBLOCK, &child_set, NULL);
-  
-  pclose(munge_pipe);
-  
-  if (bytes_read == -1)
-    {
-    /* read failed */
-    req_reject(PBSE_SYSTEM, 0, preq, NULL, "error reading unmunge data");
-    rc = -1;
-    }
-  else if (total_bytes_read == 0)
-    {
-    /* 
-     * unmunge failed. Probably a bad credential. But we do not know for sure.
-     * Bad credential gives us ECHILD error which gets added to log message
-     * and confuses users, so reset it to zero show it does not show up in log
-     */
-    if (errno == ECHILD)
-      errno = 0;
-    req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not unmunge credentials");
-    rc = -1;
-    }
-  else if ((rc = get_encode_host(sock, munge_buf, preq)) == PBSE_NONE)
-    {
-    rc = get_UID(sock, munge_buf, preq);
-    }
-
-  return(rc);
-  } /* END pipe_and_read_unmunge() */
-
-
-
-
-int unmunge_request(
+  pid = fork();
+  if (pid != 0)
+	  {
+	  /* This is the parent*/
+	  /* set up the pipe to be able to read from the child */
+	  close(fd_pipe[1]);
+	  
+	  memset(buf, 0, MUNGE_SIZE);
+	  memset(munge_buf, 0, MUNGE_SIZE);
+	  ptr = munge_buf; 
+	  
+	  do
+      {
+      bytes_read = read(fd_pipe[0], buf, MUNGE_SIZE);
+      if (bytes_read > 0)
+        {
+        total_bytes_read += bytes_read;
+        memcpy(ptr, buf, bytes_read);
+        ptr += bytes_read;
+        }
+      } while(bytes_read > 0);
     
-  int                   s,
-  struct batch_request *preq)
+	    if (bytes_read == -1)
+	  	  {
+	  	  /* read failed */
+	  	  unlink(mungeFileName);
+	  	  req_reject(PBSE_SYSTEM, 0, preq, NULL, "error reading unmunge data");
+	  		close(fd_pipe[0]);
+	  	  return(-1);
+	  	  }
+	  
+	  
+	    if (total_bytes_read == 0)
+	  	  {
+	  	  /* unmunge failed. Probably a bad credential. But we do not know */
+	  	  req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not unmunge credentials");
+				unlink(mungeFileName);
+	  		close(fd_pipe[0]);
+	  	  return(-1);
+	  	  }
+	  
+	    rc = get_encode_host(s, munge_buf, preq);
+	    if (rc)
+	  	  {
+				unlink(mungeFileName);
+	  		close(fd_pipe[0]);
+	  	  return(rc);
+	  	  }
+	  
+	    rc = get_UID(s, munge_buf, preq);
+	    if (rc)
+	    	{
+				unlink(mungeFileName);
+	  		close(fd_pipe[0]);
+    		return(rc);
+	  	  }
+	  
+	  	unlink(mungeFileName);
+	  }
+  else
+	  {
+	  close(fd_pipe[0]);
+	  newfd = dup2(fd_pipe[1], 1);
+	  strcpy(execname, "unmunge");
+	  strcpy(com1, "unmunge");
+	  strcpy(com2, "--input=");
+	  strcat(com2, mungeFileName);
+	  options[0] = com1;
+	  options[1] = com2;
+	  options[2] = NULL;
+	  
+	  rc = execvp(execname, options);
+	  
+	  /* Something went wrong. We will have to depend on the parent
+	     to let everyone know */
+	  close(fd_pipe[1]);
+	  exit(0);
+	  
+	  }
 
-  {
-  time_t          myTime;
-  struct timeval  tv;
-  suseconds_t     millisecs;
-  struct tm      *timeinfo;
-  char            mungeFileName[MAXPATHLEN + MAXNAMLEN+1];
-  int             rc = PBSE_NONE;
+	close(fd_pipe[0]);
+  return(0);
 
-  /* create a sudo random file name */
-  gettimeofday(&tv, NULL);
-  myTime = tv.tv_sec;
-  timeinfo = localtime(&myTime);
-  millisecs = tv.tv_usec;
-  sprintf(mungeFileName, "%smunge-%d-%d-%d-%d", 
-	  path_credentials, timeinfo->tm_hour, timeinfo->tm_min, 
-	  timeinfo->tm_sec, (int)millisecs);
-
-  /* Write the munge credential to the newly created file */
-  if ((rc = write_munge_temp_file(preq, mungeFileName)) == PBSE_NONE)
-    {
-    /* open the munge command as a pipe and read the result */
-    rc = pipe_and_read_unmunge(mungeFileName, preq, s);
-    }
-  
-  /* delete the old file */
-  unlink(mungeFileName);
-
-  return(rc);
-  } /* END unmunge_request() */
-
-
-/*
- * Check if the host names match
- */
-
-int check_host(struct batch_request *preq, pbs_net_t ipaddr)
-{
-  struct hostent *host;
-  struct in_addr  from;
-
-  from.s_addr = htonl(ipaddr);
-
-  if ((host = gethostbyaddr(
-                (void *) & from,
-                sizeof(struct in_addr),
-                AF_INET)) == NULL)
-    {
-    sprintf(log_buffer, "gethostbyaddr failed for host %s: h_error is %d",
-            preq->rq_host, h_errno);
-
-    log_ext(-1, "check_host", log_buffer, LOG_DEBUG);
-    return(PBSE_SYSTEM);
-    }
-
-  if (!strcmp(preq->rq_host, host->h_name))
-    {
-    return (TRUE);
-    }
-  return (FALSE);
 }
 
 
+
 /*
  * req_authenuser - Authenticate a user connection based on the (new)
  * pbs_iff information.  pbs_iff will contact the server on a privileged
@@ -446,11 +402,6 @@ void req_authenuser(
       continue;
       }
 
-    if (!check_host(preq, svr_conn[s].cn_addr))
-      {
-      continue;
-      }
-
 #ifndef NOPRIVPORTS
     if (svr_conn[s].cn_authen == 0)
 #endif
@@ -472,7 +423,7 @@ void req_authenuser(
     return;
     }  /* END for (s) */
 
-  req_reject(PBSE_BADCRED, 0, preq, NULL, "cannot authenticate user. Client connection not found");
+  req_reject(PBSE_BADCRED, 0, preq, NULL, "cannot authenticate user");
 
   /* FAILURE */
 
@@ -506,16 +457,11 @@ int req_altauthenuser(
       {
       continue;
       }
-
-    if (!check_host(preq, svr_conn[s].cn_addr))
-      {
-      continue;
-      }
-	  break;
+	break;
     }  /* END for (s) */
 
   /* If s is less than PBS_NET_MAX_CONNECTIONS we have our port */
-  if (s >= PBS_NET_MAX_CONNECTIONS)
+  if(s >= PBS_NET_MAX_CONNECTIONS)
 	  {
 	  req_reject(PBSE_BADCRED, 0, preq, NULL, "cannot authenticate user. Client connection not found");
 	  return (PBSE_BADCRED);
@@ -524,11 +470,10 @@ int req_altauthenuser(
 
   rc = unmunge_request(s, preq);
   if (rc)
-	  {
-	  /* FAILED */
-	  /*req_reject(PBSE_SYSTEM, 0, preq, NULL, "munge failure");*/
-	  return (PBSE_SYSTEM);
-  	}
+    {
+    /* FAILED */
+    return (PBSE_SYSTEM);
+    }
 
   /* SUCCESS */
 
@@ -540,6 +485,7 @@ int req_altauthenuser(
 
   reply_ack(preq);
   return (PBSE_NONE);
+
   }  /* END req_altauthenuser() */
 
 
