@@ -87,7 +87,6 @@
 #include <sys/wait.h>
 #include <signal.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <stdio.h>
 #include <netdb.h>
 #include <errno.h>
@@ -101,6 +100,7 @@
 #include "credential.h"
 #include "batch_request.h"
 #include "pbs_job.h"
+#include "pbs_nodes.h"
 #include "queue.h"
 #include "work_task.h"
 #include "pbs_error.h"
@@ -122,7 +122,7 @@ extern int                   send_job(job *, pbs_net_t, int, int, void (*x)(), s
 extern void                  set_resc_assigned(job *, enum batch_op);
 
 extern struct batch_request *cpy_stage(struct batch_request *, job *, enum job_atr, int);
-void                         stream_eof(int, u_long, int);
+void                         stream_eof(int, u_long, uint16_t, int);
 extern int                   job_set_wait(attribute *, void *, int);
 extern void                  stat_mom_job(job *);
 
@@ -166,6 +166,8 @@ long  DispatchTime[20];
 job  *DispatchJob[20];
 char *DispatchNode[20];
 
+extern job  *chk_job_request(char *, struct batch_request *);
+extern struct batch_request *cpy_checkpoint(struct batch_request *, job *, enum job_atr, int);
 
 
 /*
@@ -202,15 +204,6 @@ void req_runjob(
     return;
     }
 
-  /* we don't currently allow running of an entire job array */
-
-  if (strstr(pjob->ji_qs.ji_jobid,"[]") != NULL)
-    {
-    req_reject(PBSE_IVALREQ, 0, preq, NULL, "cannot run a job array");
-
-    return;
-    }
-
   if (preq->rq_conn == scheduler_sock)
     ++scheduler_jobct; /* see scheduler_close() */
 
@@ -298,7 +291,7 @@ static int is_checkpoint_restart(
 
   {
 #if 0
-  if ((pjob->ji_wattr[(int)JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET) == 0)
+  if ((pjob->ji_wattr[JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET) == 0)
     {
     return(FALSE);
     }
@@ -344,7 +337,7 @@ static void post_checkpointsend(
 
       free_nodes(pjob);
 
-      pwait = &pjob->ji_wattr[(int)JOB_ATR_exectime];
+      pwait = &pjob->ji_wattr[JOB_ATR_exectime];
 
       if ((pwait->at_flags & ATR_VFLAG_SET) == 0)
         {
@@ -386,14 +379,14 @@ static void post_checkpointsend(
       
       /* set restart_name attribute to the checkpoint_name we just copied */
       
-      job_attr_def[(int)JOB_ATR_restart_name].at_set(
-        &pjob->ji_wattr[(int)JOB_ATR_restart_name],
-        &pjob->ji_wattr[(int)JOB_ATR_checkpoint_name],
+      job_attr_def[JOB_ATR_restart_name].at_set(
+        &pjob->ji_wattr[JOB_ATR_restart_name],
+        &pjob->ji_wattr[JOB_ATR_checkpoint_name],
         SET);
 
       pjob->ji_modified = 1;
       
-      job_save(pjob, SAVEJOB_FULL);
+      job_save(pjob, SAVEJOB_FULL, 0);
       
       /* continue to start job running */
 
@@ -446,7 +439,7 @@ static int svr_send_checkpoint(
   strcpy(momreq->rq_extra, pjob->ji_qs.ji_jobid);
 
   rc = relay_to_mom(
-         pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
+         pjob,
          momreq,
          post_checkpointsend);
 
@@ -502,7 +495,7 @@ void req_stagein(
     return;
     }
 
-  if ((pjob->ji_wattr[(int)JOB_ATR_stagein].at_flags & ATR_VFLAG_SET) == 0)
+  if ((pjob->ji_wattr[JOB_ATR_stagein].at_flags & ATR_VFLAG_SET) == 0)
     {
     log_err(-1, "req_stagein", "stage-in information not set");
 
@@ -560,7 +553,7 @@ static void post_stagein(
 
       free_nodes(pjob);
 
-      pwait = &pjob->ji_wattr[(int)JOB_ATR_exectime];
+      pwait = &pjob->ji_wattr[JOB_ATR_exectime];
 
       if ((pwait->at_flags & ATR_VFLAG_SET) == 0)
         {
@@ -667,7 +660,7 @@ static int svr_stagein(
   strcpy(momreq->rq_extra, pjob->ji_qs.ji_jobid);
 
   rc = relay_to_mom(
-         pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
+         pjob,
          momreq,
          post_stagein);
 
@@ -732,10 +725,10 @@ int svr_startjob(
   /* if not already setup, transfer the control/script file basename */
   /* into an attribute accessible by MOM */
 
-  if (!(pjob->ji_wattr[(int)JOB_ATR_hashname].at_flags & ATR_VFLAG_SET))
+  if (!(pjob->ji_wattr[JOB_ATR_hashname].at_flags & ATR_VFLAG_SET))
     {
-    if (job_attr_def[(int)JOB_ATR_hashname].at_decode(
-          &pjob->ji_wattr[(int)JOB_ATR_hashname],
+    if (job_attr_def[JOB_ATR_hashname].at_decode(
+          &pjob->ji_wattr[JOB_ATR_hashname],
           NULL,
           NULL,
           pjob->ji_qs.ji_fileprefix))
@@ -751,7 +744,7 @@ int svr_startjob(
 
   rc = 0;
 
-  f = pjob->ji_wattr[(int)JOB_ATR_exec_host].at_flags & ATR_VFLAG_SET;
+  f = pjob->ji_wattr[JOB_ATR_exec_host].at_flags & ATR_VFLAG_SET;
 
   if ((f != 0) &&
       ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HOTSTART) ||
@@ -760,7 +753,7 @@ int svr_startjob(
     {
     rc = assign_hosts(    /* inside svr_startjob() */
            pjob,
-           pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str,
+           pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str,
            0,
            FailHost,
            EMsg);
@@ -789,7 +782,7 @@ int svr_startjob(
 
   /* NOTE: Copy the nodes into a temp string because strtok() is destructive. */
 
-  size = strlen(pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str);
+  size = strlen(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
 
   hostlist = malloc(size + 1);
 
@@ -802,7 +795,7 @@ int svr_startjob(
     {
     /* Get the first host. */
 
-    strncpy(hostlist, pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str, size);
+    strncpy(hostlist, pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, size);
     hostlist[size] = '\0';
     nodestr = strtok(hostlist, "+");
     }
@@ -914,7 +907,7 @@ int svr_startjob(
 
     memcpy(&saddr.sin_addr, hp->h_addr, hp->h_length);
 
-    saddr.sin_port = htons(pbs_rm_port);
+    saddr.sin_port = htons(pjob->ji_qs.ji_un.ji_exect.ji_mom_rmport);
 
     /* Connect to the host. */
 
@@ -977,7 +970,7 @@ int svr_startjob(
 
   /* Next, are there files to be staged-in? */
 
-  if ((pjob->ji_wattr[(int)JOB_ATR_stagein].at_flags & ATR_VFLAG_SET) &&
+  if ((pjob->ji_wattr[JOB_ATR_stagein].at_flags & ATR_VFLAG_SET) &&
       (pjob->ji_qs.ji_substate != JOB_SUBSTATE_STAGECMP))
     {
     /* yes, we do that first; then start the job */
@@ -1040,15 +1033,15 @@ static int svr_strtjob2(
   old_state = pjob->ji_qs.ji_state;
   old_subst = pjob->ji_qs.ji_substate;
 
-  pattr = &pjob->ji_wattr[(int)JOB_ATR_start_time];
+  pattr = &pjob->ji_wattr[JOB_ATR_start_time];
 
-  if ((pjob->ji_wattr[(int)JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET) == 0)
+  if ((pjob->ji_wattr[JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET) == 0)
     {
     pattr->at_val.at_long = time(NULL);
     pattr->at_flags |= ATR_VFLAG_SET;
     }
 
-  pattr = &pjob->ji_wattr[(int)JOB_ATR_start_count];
+  pattr = &pjob->ji_wattr[JOB_ATR_start_count];
 
   pattr->at_val.at_long++;
   pattr->at_flags |= ATR_VFLAG_SET;
@@ -1059,28 +1052,36 @@ static int svr_strtjob2(
 
   /* if job start timeout attribute is set use its value */
   
-  if (((server.sv_attr[(int)SRV_ATR_JobStartTimeout].at_flags & ATR_VFLAG_SET) != 0)
-          && (server.sv_attr[(int)SRV_ATR_JobStartTimeout].at_val.at_long > 0))
+  if (((server.sv_attr[SRV_ATR_JobStartTimeout].at_flags & ATR_VFLAG_SET) != 0)
+          && (server.sv_attr[SRV_ATR_JobStartTimeout].at_val.at_long > 0))
     {
-    DIS_tcp_settimeout(server.sv_attr[(int)SRV_ATR_JobStartTimeout].at_val.at_long);
+    DIS_tcp_settimeout(server.sv_attr[SRV_ATR_JobStartTimeout].at_val.at_long);
+    }
+
+  /* if job start timeout attribute is set use its value */
+  
+  if (((server.sv_attr[SRV_ATR_JobStartTimeout].at_flags & ATR_VFLAG_SET) != 0)
+          && (server.sv_attr[SRV_ATR_JobStartTimeout].at_val.at_long > 0))
+    {
+    DIS_tcp_settimeout(server.sv_attr[SRV_ATR_JobStartTimeout].at_val.at_long);
     }
 
   if (send_job(
         pjob,
         pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
-        pbs_mom_port,
+        pjob->ji_qs.ji_un.ji_exect.ji_momport,
         MOVE_TYPE_Exec,
         post_sendmom,
-        (void *)preq) == ROUTE_DEFERRED)
+        (void *)preq) == 2)
     {
     /* SUCCESS */
 
-    DIS_tcp_settimeout(server.sv_attr[(int)SRV_ATR_tcp_timeout].at_val.at_long);
+    DIS_tcp_settimeout(server.sv_attr[SRV_ATR_tcp_timeout].at_val.at_long);
 
     return(0);
     }
 
-  DIS_tcp_settimeout(server.sv_attr[(int)SRV_ATR_tcp_timeout].at_val.at_long);
+  DIS_tcp_settimeout(server.sv_attr[SRV_ATR_tcp_timeout].at_val.at_long);
 
   sprintf(tmpLine, "unable to run job, send to MOM '%s' failed",
 
@@ -1132,6 +1133,7 @@ static void post_sendmom(
   int  newstate;
   int  newsub;
   int  r;
+  ulong addr;
   int  stat;
   job *jobp = (job *)pwt->wt_parm1;
 
@@ -1242,7 +1244,7 @@ static void post_sendmom(
 
       /* if any dependencies, see if action required */
 
-      if (jobp->ji_wattr[(int)JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
+      if (jobp->ji_wattr[JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
         depend_on_exec(jobp);
 
       /*
@@ -1262,7 +1264,8 @@ static void post_sendmom(
 
       /* NOTE: if r == 10, connection to mom timed out.  Mark node down */
 
-      stream_eof(-1, jobp->ji_qs.ji_un.ji_exect.ji_momaddr, 0);
+      addr = jobp->ji_qs.ji_un.ji_exect.ji_momaddr;
+      stream_eof(-1, addr, jobp->ji_qs.ji_un.ji_exect.ji_momport, 0);
 
       /* send failed, requeue the job */
 
@@ -1461,7 +1464,7 @@ static job *chk_job_torun(
       {
       /* specified destination must match exec_host */
 
-      strcpy(exec_host, pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str);
+      strcpy(exec_host, pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
 
       if ((ptr = strchr(exec_host, '/')))
         * ptr = 0; /* For some reason, node name has "/0" on the end (i.e. "node0001/0"). */
@@ -1485,7 +1488,7 @@ static job *chk_job_torun(
 
       if ((rc = assign_hosts(  /* inside chk_job_torun() */
                   pjob,
-                  pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str,
+                  pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str,
                   0,
                   FailHost,
                   EMsg)) != 0)   /* O */
@@ -1542,7 +1545,7 @@ static job *chk_job_torun(
 
     attribute *Attr;     /* 'neednodes' attribute */
 
-    Attr = &pjob->ji_wattr[(int)JOB_ATR_resource];
+    Attr = &pjob->ji_wattr[JOB_ATR_resource];
 
     DRes = find_resc_def(svr_resc_def, "neednodes", svr_resc_size);
 
@@ -1578,166 +1581,47 @@ static job *chk_job_torun(
 
 
 
-
-#ifdef NVIDIA_GPUS
-char *get_correct_spec_string(
-
-  char *given,
-  job  *pjob)
-
+/* set_mother_superior_ports - The first host in list is the host
+   of Mother Superior. Find the mom manager and service ports
+   from the pbsndlist and then set the pjob mom ports accordingly */
+int set_mother_superior_ports(job *pjob, char *list)
   {
-  static char   id[] = "get_correct_spec_string";
-  char     mode[20];
-  char    *mode_string;
-  char    *request;
-  char    *correct_spec;
-  char    *outer_plus;
-  char    *plus;
-  char    *one_req;
-  int      num_gpu_reqs;
-  char    *gpu_req;
-  int      len;
-  resource *pres;
+  char ms[PBS_MAXHOSTNAME];
+  char *ptr;
+  int  i;
+  struct pbsnode *pnode;
 
-  /* check to see if there is a gpus request. If so moab
-   * sripted the mode request if it existed. We need to
-   * put it back */
-  mode_string = strstr(given, ":gpus=");
+  node_iterator *iter;
 
-  if (mode_string != NULL)
+  if (list == NULL)
     {
-    /* Build our host list from what is in the job attrs */
-    pres = find_resc_entry(
-      &pjob->ji_wattr[(int)JOB_ATR_resource],
-      find_resc_def(svr_resc_def, "neednodes", svr_resc_size));
-    
-    if (pres != NULL)
+    return(PBSE_UNKNODEATR);
+    }
+
+  memset(ms, 0, PBS_MAXHOSTNAME);
+  ptr = list;
+
+  /* get the first name in list. This is Mother Superior */
+  for(i = 0; ptr && (*ptr != '/') && (i < PBS_MAXHOSTNAME); i++)
+    {
+    ms[i] = *ptr;
+    ptr++;
+    }
+
+  iter = get_node_iterator();
+
+  while ((pnode = next_node(iter)) != NULL)
+    {
+    if (!strcasecmp(pnode->nd_name, ms))
       {
-      /* determine # of gpu requests in spec, we found 1 in given up above */
-      num_gpu_reqs = 1;
-      gpu_req = mode_string;
-      while ((gpu_req = strstr(gpu_req + 1, ":gpus=")) != NULL)
-        num_gpu_reqs++;
-
-      /* assign gpu mode that was in "neednodes" */
-      request = pres->rs_value.at_val.at_str;
-      
-      if ((request != NULL) && 
-          (request[0] != 0))
-        {
-        gpu_req = strstr(request, ":gpus=");
-
-        mode_string = gpu_req + strlen(":gpus=");
-        while (isdigit(*mode_string))
-          mode_string++;
-
-        if (*mode_string == ':')
-          {
-          if (LOGLEVEL >= 7)
-            {
-            sprintf(log_buffer, "%s: job has %d gpu requests in node spec '%s'",
-                    id,
-                    num_gpu_reqs,
-                    given);
-
-            log_event(
-              PBSEVENT_JOB,
-              PBS_EVENTCLASS_JOB,
-              pjob->ji_qs.ji_jobid,
-              log_buffer);
-            }
-
-          if ((outer_plus = strchr(mode_string, '+')) != NULL)
-            *outer_plus = '\0';
-
-          /* 
-           * The neednodes original value may have non gpu things in it, so we
-           * can not rely on the requested gpu mode being the first item in the
-           * the string after the gpus=x:.
-           */
-
-          if (strstr(mode_string, "exclusive_thread"))
-            {
-            strcpy(mode, ":exclusive_thread");
-            }
-          else if (strstr(mode_string, "exclusive_process"))
-            {
-            strcpy(mode, ":exclusive_process");
-            }
-          else if (strstr(mode_string, "exclusive"))
-            {
-            strcpy(mode, ":exclusive");
-            }
-          else if (strstr(mode_string, "default"))
-            {
-            strcpy(mode, ":default");
-            }
-          else if (strstr(mode_string, "shared"))
-            {
-            strcpy(mode, ":shared");
-            }
-          else
-            {
-            strcpy(mode, "");
-            }
-
-          if (outer_plus != NULL)
-            *outer_plus = '+';
-
-          /* now using the actual length of requested gpu mode */
-          len = strlen(given) + 1 + (num_gpu_reqs * strlen(mode));
-          if ((correct_spec = calloc(1, len)) != NULL)
-            {
-            one_req = given;
-            
-            while (one_req != NULL)
-              {
-              if ((plus = strchr(one_req, '+')) != NULL)
-                {
-                *plus = '\0';
-                }
-              
-              strcat(correct_spec, one_req);
-              if (strstr(one_req, ":gpus") != NULL)
-                strcat(correct_spec, mode);
-              
-              if (plus != NULL)
-                {
-                strcat(correct_spec, "+");
-                one_req = plus + 1;
-                }
-              else
-                one_req = NULL;
-              }
-            }
-          if ((LOGLEVEL >= 7) && (correct_spec != NULL) && (correct_spec[0] != '\0'))
-            {
-            sprintf(log_buffer, "%s: job gets adjusted gpu node spec of '%s'",
-                    id,
-                    correct_spec);
-
-            log_event(
-              PBSEVENT_JOB,
-              PBS_EVENTCLASS_JOB,
-              pjob->ji_qs.ji_jobid,
-              log_buffer);
-            }
-          }
-        else
-          correct_spec = strdup(given);
-        }
+      pjob->ji_qs.ji_un.ji_exect.ji_momport = pnode->nd_mom_port;
+      pjob->ji_qs.ji_un.ji_exect.ji_mom_rmport = pnode->nd_mom_rm_port;
+      return(PBSE_NONE);
       }
     }
-  else
-    correct_spec = strdup(given);
-
-  return(correct_spec);
-  } /* get_correct_spec_string() */
-#endif
-
-
-
 
+  return(PBSE_UNKNODEATR);
+  }
 
 
 /*
@@ -1759,12 +1643,12 @@ static int assign_hosts(
   {
   unsigned int  dummy;
   char  *list = NULL;
+  char  *portlist = NULL;
   char  *hosttoalloc = NULL;
   pbs_net_t  momaddr = 0;
+  resource *pres;
   int   rc = 0, procs=0;
   extern char  *mom_host;
-  char *to_free = NULL;
-  resource *pres;
 
 
   if (EMsg != NULL)
@@ -1786,18 +1670,44 @@ static int assign_hosts(
   if ((given != NULL) && (given[0] != '\0'))
     {
 #ifdef NVIDIA_GPUS
-    hosttoalloc = get_correct_spec_string(given, pjob);
-    to_free = hosttoalloc;
-#else
+    char *mode_string;
+    char *request;
+#endif
     /* assign what was specified in run request */
+
     hosttoalloc = given;
+
+    /* check to see if there is a gpus request. If so moab
+     * sripted the mode request if it existed. We need to
+     * put it back */
+#ifdef NVIDIA_GPUS
+    mode_string = strstr(hosttoalloc, ":gpus=");
+    if(mode_string != NULL)
+      {
+      /* Build our host list from what is in the job attrs */
+      pres = find_resc_entry(
+               &pjob->ji_wattr[(int)JOB_ATR_resource],
+               find_resc_def(svr_resc_def, "neednodes", svr_resc_size));
+
+      if (pres != NULL)
+        {
+        /* assign what was in "neednodes" */
+
+        request = pres->rs_value.at_val.at_str;
+
+        if(request != NULL && request[0] != 0)
+          {
+          hosttoalloc = request;
+          }
+        }
+      }
 #endif
     }
   else
     {
     /* Build our host list from what is in the job attrs */
     pres = find_resc_entry(
-             &pjob->ji_wattr[(int)JOB_ATR_resource],
+             &pjob->ji_wattr[JOB_ATR_resource],
              find_resc_def(svr_resc_def, "neednodes", svr_resc_size));
 
     if (pres != NULL)
@@ -1813,7 +1723,7 @@ static int assign_hosts(
       }
 
     pres = find_resc_entry(
-             &pjob->ji_wattr[(int)JOB_ATR_resource],
+             &pjob->ji_wattr[JOB_ATR_resource],
              find_resc_def(svr_resc_def, "procs", svr_resc_size));
 
     if (pres != NULL)
@@ -1837,10 +1747,10 @@ static int assign_hosts(
     {
     /* assign "local" */
 
-    if ((server.sv_attr[(int)SRV_ATR_DefNode].at_flags & ATR_VFLAG_SET) &&
-        (server.sv_attr[(int)SRV_ATR_DefNode].at_val.at_str != NULL))
+    if ((server.sv_attr[SRV_ATR_DefNode].at_flags & ATR_VFLAG_SET) &&
+        (server.sv_attr[SRV_ATR_DefNode].at_val.at_str != NULL))
       {
-      hosttoalloc = server.sv_attr[(int)SRV_ATR_DefNode].at_val.at_str;
+      hosttoalloc = server.sv_attr[SRV_ATR_DefNode].at_val.at_str;
       }
     else
       {
@@ -1848,12 +1758,12 @@ static int assign_hosts(
       momaddr = pbs_mom_addr;
       }
     }
-  else if ((server.sv_attr[(int)SRV_ATR_DefNode].at_flags & ATR_VFLAG_SET) &&
-           (server.sv_attr[(int)SRV_ATR_DefNode].at_val.at_str != 0))
+  else if ((server.sv_attr[SRV_ATR_DefNode].at_flags & ATR_VFLAG_SET) &&
+           (server.sv_attr[SRV_ATR_DefNode].at_val.at_str != 0))
     {
     /* alloc server default_node */
 
-    hosttoalloc = server.sv_attr[(int)SRV_ATR_DefNode].at_val.at_str;
+    hosttoalloc = server.sv_attr[SRV_ATR_DefNode].at_val.at_str;
     }
   else if (svr_tsnodes != 0)
     {
@@ -1879,7 +1789,7 @@ static int assign_hosts(
     {
     if ((rc = is_ts_node(hosttoalloc)) != 0)
       {
-      rc = set_nodes(pjob, hosttoalloc, procs, &list, FailHost, EMsg);
+      rc = set_nodes(pjob, hosttoalloc, procs, &list, &portlist, FailHost, EMsg);
 
       set_exec_host = 1; /* maybe new VPs, must set */
 
@@ -1893,23 +1803,34 @@ static int assign_hosts(
 
     if (set_exec_host != 0)
       {
-      job_attr_def[(int)JOB_ATR_exec_host].at_free(
-        &pjob->ji_wattr[(int)JOB_ATR_exec_host]);
+      job_attr_def[JOB_ATR_exec_host].at_free(
+        &pjob->ji_wattr[JOB_ATR_exec_host]);
 
-      job_attr_def[(int)JOB_ATR_exec_host].at_decode(
-        &pjob->ji_wattr[(int)JOB_ATR_exec_host],
+      job_attr_def[JOB_ATR_exec_host].at_decode(
+        &pjob->ji_wattr[JOB_ATR_exec_host],
         NULL,
         NULL,
         hosttoalloc);  /* O */
 
+      job_attr_def[JOB_ATR_exec_port].at_free(
+          &pjob->ji_wattr[JOB_ATR_exec_port]);
+
+      job_attr_def[JOB_ATR_exec_port].at_decode(
+        &pjob->ji_wattr[JOB_ATR_exec_port],
+        NULL,
+        NULL,
+        portlist);  /* O */
+
       pjob->ji_modified = 1;
       }
     else
       {
       /* leave exec_host alone and reuse old IP address */
+
       momaddr = pjob->ji_qs.ji_un.ji_exect.ji_momaddr;
 
-      hosttoalloc = pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str;
+      hosttoalloc = pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str;
+      portlist = pjob->ji_wattr[JOB_ATR_exec_port].at_val.at_str;
       }
 
     strncpy(
@@ -1938,21 +1859,21 @@ static int assign_hosts(
           pjob->ji_qs.ji_jobid,
           log_buffer);
 
-        if (to_free != NULL)
-          free(to_free);
-
         return(PBSE_BADHOST);
         }
       }
 
     pjob->ji_qs.ji_un.ji_exect.ji_momaddr = momaddr;
+
+    rc = set_mother_superior_ports(pjob, list);
+
     }  /* END if (rc == 0) */
 
   if (list != NULL)
     free(list);
-        
-  if (to_free != NULL)
-    free(to_free);
+
+  if (portlist != NULL)
+    free(portlist);
 
   return(rc);
   }  /* END assign_hosts() */
