@@ -165,6 +165,7 @@ extern int  svr_clnodes;
 extern int node_avail_complex(char *, int *, int *, int *, int *);
 extern int procs_available(int proc_ct);
 extern int procs_requested(char *spec);
+extern int remove_procct(job *);
 
 
 /* Private Functions */
@@ -827,10 +828,6 @@ void svr_evaljobstate(
     *newsub   = JOB_SUBSTATE_QUEUED;
     }
 
-  if (pjob->ji_is_array_template)
-    {
-    *newsub = JOB_SUBSTATE_ARRAY_TEMP;
-    }
   return;
   }  /* END svr_evaljobstate() */
 
@@ -1301,7 +1298,7 @@ static void chk_svr_resc_limit(
             (!pque->qu_attr[QE_ATR_is_transit].at_val.at_long))
           {
           if ((EMsg != NULL) && (EMsg[0] == '\0'))
-            strcpy(EMsg, "cannot locate feasible nodes");
+            strcpy(EMsg, "cannot locate feasible nodes (nodes file is empty or all systems are busy)");
         
           comp_resc_lt++;
           }
@@ -1321,7 +1318,7 @@ static void chk_svr_resc_limit(
               (!pque->qu_attr[QE_ATR_is_transit].at_val.at_long))
             {
             if ((EMsg != NULL) && (EMsg[0] == '\0'))
-              strcpy(EMsg, "cannot locate feasible nodes");
+              strcpy(EMsg, "cannot locate feasible nodes (nodes file is empty or requested nodes exceed system)");
       
             comp_resc_lt++;
             }
@@ -1334,7 +1331,7 @@ static void chk_svr_resc_limit(
         (!pque->qu_attr[QE_ATR_is_transit].at_val.at_long))
       {
       if ((EMsg != NULL) && (EMsg[0] == '\0'))
-        strcpy(EMsg, "cannot locate feasible nodes");
+        strcpy(EMsg, "cannot locate feasible nodes (nodes file is empty or requested nodes exceed all systems)");
     
       comp_resc_lt++;
       }
@@ -2399,8 +2396,6 @@ void set_resc_deflt(
   attribute *ji_wattr) /* I (optional) decoded attributes  */
 
   {
-  resource_def  *pctdef;
-  resource      *pctresc;
   attribute     *ja;
 
   pbs_queue     *pque;
@@ -2456,10 +2451,7 @@ void set_resc_deflt(
   if (pque->qu_qs.qu_type == QTYPE_Execution)
     {
     /* unset the procct resource if it has been set */
-    pctdef = find_resc_def(svr_resc_def, "procct", svr_resc_size);
- 
-    if ((pctresc = find_resc_entry(ja, pctdef)) != NULL)
-      pctdef->rs_free(&pctresc->rs_value);
+    remove_procct(pjob);
     }
  
   return;
