@@ -109,6 +109,7 @@
 #include "pbs_error.h"
 #include "resource.h"
 #include "utils.h"
+#include "dynamic_string.h"
 
 #ifndef MAXLINE
 #define MAXLINE 1024
@@ -329,20 +330,23 @@ int size_to_str(
 /*
  * attr_to_str
  *
- * @param out - the string output
- * @param size - the largest possible string length
+ * @param ds - the dynamic string we're printing the attribute into
  * @param aindex - the attribute's index
  * @param attr - the attribute
+ * @param XML - boolean telling us whether to print XML or not
  */
 int attr_to_str(
 
-  char             *out,    /* O */
-  int               size,   /* I */
+  dynamic_string   *ds,     /* O */
   attribute_def    *at_def, /* I */
   struct attribute  attr,   /* I */
   int               XML)    /* I */
 
   {
+  int rc = 0;
+  /* used to print numbers and chars as strings */
+  char local_buf[MAXLINE];
+
   if ((attr.at_flags & ATR_VFLAG_SET) == FALSE)
     return(NO_ATTR_DATA);
 
@@ -350,21 +354,15 @@ int attr_to_str(
     {
     case ATR_TYPE_LONG:
 
-      snprintf(out,size,"%ld",attr.at_val.at_long);
+      snprintf(local_buf, sizeof(local_buf), "%ld", attr.at_val.at_long);
+      rc = append_dynamic_string(ds, local_buf);
 
       break;
 
     case ATR_TYPE_CHAR:
 
-      if (size > 1)
-        {
-        out[0] = attr.at_val.at_char;
-        out[1] = '\0';
-        }
-      else
-        {
-        return(NO_BUFFER_SPACE);
-        }
+      sprintf(local_buf, "%c", attr.at_val.at_char);
+      rc = append_dynamic_string(ds, local_buf);
 
       break;
 
@@ -377,9 +375,9 @@ int attr_to_str(
         return(NO_ATTR_DATA);
 
       if (XML)
-        escape_xml(attr.at_val.at_str,out,size);
+        rc = append_dynamic_string_xml(ds, attr.at_val.at_str);
       else
-        snprintf(out,size,"%s",attr.at_val.at_str);
+        rc = append_dynamic_string(ds, attr.at_val.at_str);
 
       break;
 
@@ -393,36 +391,16 @@ int attr_to_str(
       if (arst == NULL)
         return(NO_ATTR_DATA);
 
-      if (size < arst->as_bufsize)
-        return(NO_BUFFER_SPACE);
-
-      out[0] = '\0';
-
       /* concatenate all of the array strings into one string */
       for (j = 0; j < arst->as_usedptr; j++)
         {
         if (j > 0)
-          {
-          int len;
-          strcat(out,",");
-          if (XML)
-            {
-            len = strlen(out);
-            escape_xml(arst->as_string[j],out+len,size-len);
-            }
-          else
-            strcat(out,arst->as_string[j]);
-          }
+          append_dynamic_string(ds, ",");
+
+        if (XML)
+          rc = append_dynamic_string_xml(ds, arst->as_string[j]);
         else
-          {
-          if (XML) 
-            {
-            int len = strlen(out);
-            escape_xml(arst->as_string[j],out+len,size-len);
-            }
-          else
-            strcat(out,arst->as_string[j]);
-          }
+          rc = append_dynamic_string(ds, arst->as_string[j]);
         }
       }
 
@@ -430,43 +408,36 @@ int attr_to_str(
 
     case ATR_TYPE_SIZE:
 
-      size_to_str(attr.at_val.at_size,out,size);
+      rc = size_to_dynamic_string(ds, attr.at_val.at_size);
 
       break;
 
     case ATR_TYPE_RESC:
 
       {
-      char *ptr;
-
-      int   lspace;
-      int   len;
-
       resource *current = (resource *)GET_NEXT(attr.at_val.at_list);
 
       if (current == NULL)
         return(NO_ATTR_DATA);
 
-      ptr = out;
-      lspace = size;
-
       /* print all of the resources */
       while (current != NULL)
         {
-
         /* there are only 3 resource types used */
         switch (current->rs_value.at_type)
           {
           case ATR_TYPE_LONG:
 
-            snprintf(ptr,lspace,"\t\t<%s>%ld</%s>\n",
-              current->rs_defin->rs_name,
-              current->rs_value.at_val.at_long,
-              current->rs_defin->rs_name);
-            
-            len = strlen(ptr);
-            ptr    += len;
-            lspace -= len;
+            append_dynamic_string(ds, "\t\t<");
+            append_dynamic_string(ds, current->rs_defin->rs_name);
+            append_dynamic_string(ds, ">");
+
+            snprintf(local_buf, sizeof(local_buf), "%ld", current->rs_value.at_val.at_long);
+            append_dynamic_string(ds, local_buf);
+
+            append_dynamic_string(ds, "</");
+            append_dynamic_string(ds, current->rs_defin->rs_name);
+            rc = append_dynamic_string(ds, ">\n");
 
             break;
 
@@ -477,57 +448,44 @@ int attr_to_str(
              * Bugzilla bug 101 
              */
 
-            if(current->rs_value.at_val.at_str == NULL)
+            if (current->rs_value.at_val.at_str == NULL)
               break;
 
-            if(strlen(current->rs_value.at_val.at_str) == 0)
+            if (strlen(current->rs_value.at_val.at_str) == 0)
               break;
 
-            snprintf(ptr,lspace,"\t\t<%s>",
-              current->rs_defin->rs_name);
-            len = strlen(ptr);
-            ptr    += len;
-            lspace -= len;
+            append_dynamic_string(ds, "\t\t<");
+            append_dynamic_string(ds, current->rs_defin->rs_name);
+            append_dynamic_string(ds, ">");
 
             if (XML)
-              escape_xml(current->rs_value.at_val.at_str,ptr,lspace);
+              append_dynamic_string_xml(ds, current->rs_value.at_val.at_str);
             else
-              snprintf(ptr,lspace,"%s",current->rs_value.at_val.at_str);
+              append_dynamic_string(ds, current->rs_value.at_val.at_str);
 
-            len = strlen(ptr);
-            ptr += len;
-            lspace -= len;
-
-            snprintf(ptr,lspace,"</%s>\n",
-              current->rs_defin->rs_name);
-            len = strlen(ptr);
-            ptr += len;
-            lspace -= len;
+            append_dynamic_string(ds, "</");
+            append_dynamic_string(ds, current->rs_defin->rs_name);
+            rc = append_dynamic_string(ds, ">\n");
 
             break;
 
           case ATR_TYPE_SIZE:
 
-            snprintf(ptr,lspace,"\t\t<%s>",current->rs_defin->rs_name);
-            len = strlen(ptr);
-            ptr += len;
-            lspace -= len;
-
-            size_to_str(current->rs_value.at_val.at_size,ptr,size);
-            len = strlen(ptr);
-            ptr += len;
-            lspace -= len;
-
-            snprintf(ptr,lspace,"</%s>\n",current->rs_defin->rs_name);
-            len = strlen(ptr);
-            ptr += len;
-            lspace -= len;
+            append_dynamic_string(ds, "\t\t<");
+            append_dynamic_string(ds, current->rs_defin->rs_name);
+            append_dynamic_string(ds, ">");
+ 
+            size_to_dynamic_string(ds, current->rs_value.at_val.at_size);
+ 
+            append_dynamic_string(ds, "</");
+            append_dynamic_string(ds, current->rs_defin->rs_name);
+            rc = append_dynamic_string(ds, ">\n");
 
             break;
           }
 
-        if (lspace == 0)
-          return(NO_BUFFER_SPACE);
+        if (rc != PBSE_NONE)
+          return(rc);
 
         current = (resource *)GET_NEXT(current->rs_link);
         }
@@ -545,18 +503,22 @@ int attr_to_str(
       break;
     } /* END switch attribute type */
 
-  return(0);
+  return(PBSE_NONE);
   } /* END attr_to_str */
 
 
 
 
+/* converts a string to an attribute 
+ * @return PBSE_NONE on success
+ */
 
 int str_to_attr(
 
-  char             *name,
-  char             *val,
-  struct attribute *attr)
+  char                *name,    /* I */
+  char                 *val,    /* I */
+  struct attribute     *attr,   /* O */
+  struct attribute_def *padef)  /* I */
 
   {
   int   index;
@@ -572,7 +534,7 @@ int str_to_attr(
     return(-10);
     }
 
-  index = find_attr(svr_attr_def,name,SRV_ATR_LAST);
+  index = find_attr(padef,name,SRV_ATR_LAST);
 
   if (index < 0)
     {
@@ -585,7 +547,7 @@ int str_to_attr(
     return(ATTR_NOT_FOUND);
     }
 
-  switch (svr_attr_def[index].at_type)
+  switch (padef[index].at_type)
     {
     case ATR_TYPE_LONG:
 
@@ -709,7 +671,7 @@ int str_to_attr(
           break;
           }
         
-        if ((rc = decode_resc(&(server.sv_attr[index]),name,resc_parent,resc_child)))
+        if ((rc = decode_resc(&(attr[index]),name,resc_parent,resc_child)))
           {
           snprintf(log_buffer,sizeof(log_buffer),
             "Error decoding resource %s, %s = %s\n",
@@ -741,7 +703,7 @@ int str_to_attr(
 
   attr[index].at_flags |= ATR_VFLAG_SET;
 
-  return(0);
+  return(PBSE_NONE);
   } /* END str_to_attr */
 
 
@@ -830,7 +792,7 @@ int svr_recov_xml(
     }
 
   /* adjust to not process server tag */
-  current += strlen("<server>");
+  current += strlen("<server_db>");
   /* adjust end for the newline character preceeding the close server tag */
   end--;
 
@@ -879,7 +841,7 @@ int svr_recov_xml(
           break;
           }
 
-        if ((rc = str_to_attr(child_parent,child_attr,server.sv_attr)))
+        if ((rc = str_to_attr(child_parent,child_attr,server.sv_attr,svr_attr_def)))
           {
           /* ERROR */
           errorCount++;
@@ -901,10 +863,10 @@ int svr_recov_xml(
     
   if (!read_only)
     {
-    server.sv_attr[(int)SRV_ATR_NextJobNumber].at_val.at_long = 
+    server.sv_attr[SRV_ATR_NextJobNumber].at_val.at_long = 
       server.sv_qs.sv_jobidnumber;
     
-    server.sv_attr[(int)SRV_ATR_NextJobNumber].at_flags |= 
+    server.sv_attr[SRV_ATR_NextJobNumber].at_flags |= 
       ATR_VFLAG_SET| ATR_VFLAG_MODIFY;
     }
 
@@ -924,14 +886,12 @@ int svr_save_xml(
   int            mode)
 
   {
-  char *id   = "svr_save_xml";
+  char *id = "svr_save_xml";
   char  buf[MAXLINE<<8];
-  char  valbuf[MAXLINE<<7];
 
   int   fds;
   int   rc;
   int   len;
-  int   i;
 
   fds = open(path_svrdb, O_WRONLY | O_CREAT | O_Sync | O_TRUNC, 0600);
 
@@ -954,55 +914,7 @@ int svr_save_xml(
   if ((rc = write_buffer(buf,len,fds)))
     return(rc);
 
-  /* write the attribute info */
-  snprintf(buf,sizeof(buf),"<attributes>");
-  if ((rc = write_buffer(buf,strlen(buf),fds)))
-    return(rc);
-
-  i = 0;
-  while (i != SRV_ATR_LAST)
-    {
-    if (ps->sv_attr[i].at_flags & ATR_VFLAG_SET)
-      {
-      buf[0] = '\0';
-      valbuf[0] = '\0';
-      if ((rc = attr_to_str(valbuf,
-              sizeof(valbuf),
-              svr_attr_def + i, 
-              ps->sv_attr[i],
-              TRUE) != 0))
-        {
-        if (rc != NO_ATTR_DATA)
-          {
-          /* ERROR */
-          snprintf(log_buffer,sizeof(log_buffer),
-            "Not enough space to print attribute %s",
-            svr_attr_def[i].at_name);
-          log_err(-1,id,log_buffer);
-
-          return(rc);
-          }
-        }
-      else
-        {
-        snprintf(buf,sizeof(buf),"<%s>%s</%s>\n",
-          svr_attr_def[i].at_name,
-          valbuf,
-          svr_attr_def[i].at_name);
-        
-        if (buf[0] != '\0')
-          {
-          if ((rc = write_buffer(buf,strlen(buf),fds)))
-            return(rc);
-          }
-        }      
-      }
-
-    i++;
-    }
-  
-  snprintf(buf,sizeof(buf),"</attributes>\n");
-  if ((rc = write_buffer(buf,strlen(buf),fds)))
+  if ((rc = save_attr_xml(svr_attr_def,ps->sv_attr,SRV_ATR_LAST,fds)) != 0)
     return(rc);
 
   /* close the server_db */
@@ -1013,7 +925,6 @@ int svr_save_xml(
   close(fds);
 
   return(0);
-
   } /* END svr_save_xml */
 
 
